# NOTE: Derived from blib/lib/ClearCase/Wrapper.pm.
# Changes made here will be lost when autosplit again.
# See AutoSplit.pm.
package ClearCase::Wrapper;

#line 999 "blib/lib/ClearCase/Wrapper.pm (autosplit into blib/lib/auto/ClearCase/Wrapper/mkview.al)"
sub mkview {
    # For now at least, take the easy way out on Windows.
    die Msg('E', "please use the GUI to make views on Windows!") if MSWIN;

    # Policy: print warnings if new view's tag does not match the
    # local naming convention, or if the storage location is not
    # one of the approved areas.
    # Extension: if no view-storage area specified, use a standard one.
    my %opt;
    GetOptions(\%opt, qw(local clone=s));

    if (!grep(/^-sna/, @ARGV)) {
	my $gstg;

	# Site-specific preference for where we like to locate our views.
	# If there's a local /*/vwstore area which is shared and automountable,
	# put the view there. Otherwise require an explicit choice.
	# This array holds (<global> <local>) storage path pairs.
	my $vhost = 'sparc5';
	my @vwsmap = qw(/data/ccase/vwstore/personal
						/data/ccase/vwstore/personal);
	if ($opt{local}) {
	    warn Msg('W', "flag not implemented, no automounter in use");
	}

	{
	    local(@ARGV) = @ARGV;	# operate on temp argv
	    my %ignore;
	    GetOptions(\%ignore, qw(ncaexported|shareable_dos|nshareable_dos
			tmode|region|ln|host|hpath|gpath|cachesize|stream=s));
	    GetOptions(\%opt, q(tag=s));
	    return if !$opt{tag};
	    if ($opt{tag} && ($#ARGV == 0) && @vwsmap) {
		$gstg = "$vwsmap[0]/$ENV{LOGNAME}/$opt{tag}.vws";
	    }
	}
	if ($gstg) {
	    my $lstg = "$vwsmap[1]/$ENV{LOGNAME}/$opt{tag}.vws";
	    push(@ARGV, '-gpa', $gstg, '-hpa', $lstg, '-host', $vhost, $gstg);
	}

	if ($opt{tag}) {
	    # Policy: view-storage areas should be in a std place.
	    if (@vwsmap) {
		my $stgpat = "$ENV{LOGNAME}/$opt{tag}.vws";
		if ($ARGV[-1] =~ m%$stgpat$%) {
		    my($vwbase) = ($ARGV[-1] =~ m%(.+)/[^/]+\.vws$%);
		    mkpath($vwbase, 0, 0755) unless -d $vwbase;
		} else {
		    warn Msg("standard view storage path is /vws/.../$stgpat");
		}
	    }
	}
    }

    # Policy: users' views should be prefixed by username.
    warn Msg('W', "personal view names should match $ENV{LOGNAME}_*")
	if !grep(/^$ENV{LOGNAME}$/, @Admins) && $opt{tag} !~ /^$ENV{LOGNAME}_/;

    # If an option was used requiring a special config spec, make the
    # view here, change the cspec, then exit. Must be done this way
    # because mkview provides no way to specify the initial config spec.
    # Also clone other properties such as cache size and text mode.
    if ($opt{clone}) {
	my $ct = ClearCase::Argv->new;
	chomp(my @data = $ct->lsview([qw(-prop -full)], $opt{clone})->qx);
	my %lsview = map {(split /:\s*/)[0,1]} @data;
	splice(@ARGV, 1, 0, '-tmode', $lsview{'Text mode'})
		   if $lsview{'Text mode'};
	my %properties = map {$_ => 1} split(/\s+/, $lsview{Properties});
	for (keys %properties) { splice(@ARGV, 1, 0, "-$_") if /shareable_do/ }
	my($cachebytes) = map {(split /\s+/)[0]} map {(split /:\s*/)[1]}
		    reverse $ct->getcache([qw(-view -s)], $opt{clone})->qx;
	splice(@ARGV, 1, 0, '-cachesize', $cachebytes);
	$ct->autofail(1);
	$ct->argv(@ARGV)->system;
	my $cstmp = ".mkview.$$.cs.$opt{tag}";
	my $ctx = ClearCase::Argv->cleartool;
	Argv->new("$ctx catcs -tag $opt{clone} > $cstmp")->system;
	$ct->setcs('-tag', $opt{tag}, $cstmp)->system;
	unlink($cstmp);
	exit 0;
    }

    # Generally speaking user's views should be -nshareable while
    # standard nightly-build views should be -shareable. Since by
    # convention the former are prefixed with username, we make
    # the shareability default contingent on that while always
    # allowing a literal flag to win.
    if (!grep /^-(?:sha|nsh)/, @ARGV) {
	splice(@ARGV, 1, 0, ($opt{tag} =~ /^$ENV{LOGNAME}_/) ? '-nsh' : '-sha');
    }

    return 0;
}

# end of ClearCase::Wrapper::mkview
1;
